/****************************************************************************
** Meta object code from reading C++ file 'qwseat.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwseat.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwseat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7qw_seatE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_seat::qt_create_metaobjectdata<qt_meta_tag_ZN7qw_seatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_seat",
        "notify_pointer_grab_begin",
        "",
        "notify_pointer_grab_end",
        "notify_keyboard_grab_begin",
        "notify_keyboard_grab_end",
        "notify_touch_grab_begin",
        "notify_touch_grab_end",
        "notify_request_set_cursor",
        "wlr_seat_pointer_request_set_cursor_event*",
        "notify_request_set_selection",
        "wlr_seat_request_set_selection_event*",
        "notify_set_selection",
        "notify_request_set_primary_selection",
        "wlr_seat_request_set_primary_selection_event*",
        "notify_set_primary_selection",
        "notify_request_start_drag",
        "wlr_seat_request_start_drag_event*",
        "notify_start_drag",
        "wlr_drag*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_pointer_grab_begin'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_pointer_grab_end'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_keyboard_grab_begin'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_keyboard_grab_end'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_touch_grab_begin'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_touch_grab_end'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_set_cursor'
        QtMocHelpers::SignalData<void(wlr_seat_pointer_request_set_cursor_event *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'notify_request_set_selection'
        QtMocHelpers::SignalData<void(wlr_seat_request_set_selection_event *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Signal 'notify_set_selection'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_set_primary_selection'
        QtMocHelpers::SignalData<void(wlr_seat_request_set_primary_selection_event *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Signal 'notify_set_primary_selection'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_start_drag'
        QtMocHelpers::SignalData<void(wlr_seat_request_start_drag_event *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 2 },
        }}),
        // Signal 'notify_start_drag'
        QtMocHelpers::SignalData<void(wlr_drag *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_seat, qt_meta_tag_ZN7qw_seatE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_seat::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_seat,qw_seat>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7qw_seatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7qw_seatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7qw_seatE_t>.metaTypes,
    nullptr
} };

void qw_seat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_seat *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_pointer_grab_begin(); break;
        case 1: _t->notify_pointer_grab_end(); break;
        case 2: _t->notify_keyboard_grab_begin(); break;
        case 3: _t->notify_keyboard_grab_end(); break;
        case 4: _t->notify_touch_grab_begin(); break;
        case 5: _t->notify_touch_grab_end(); break;
        case 6: _t->notify_request_set_cursor((*reinterpret_cast<std::add_pointer_t<wlr_seat_pointer_request_set_cursor_event*>>(_a[1]))); break;
        case 7: _t->notify_request_set_selection((*reinterpret_cast<std::add_pointer_t<wlr_seat_request_set_selection_event*>>(_a[1]))); break;
        case 8: _t->notify_set_selection(); break;
        case 9: _t->notify_request_set_primary_selection((*reinterpret_cast<std::add_pointer_t<wlr_seat_request_set_primary_selection_event*>>(_a[1]))); break;
        case 10: _t->notify_set_primary_selection(); break;
        case 11: _t->notify_request_start_drag((*reinterpret_cast<std::add_pointer_t<wlr_seat_request_start_drag_event*>>(_a[1]))); break;
        case 12: _t->notify_start_drag((*reinterpret_cast<std::add_pointer_t<wlr_drag*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_pointer_grab_begin, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_pointer_grab_end, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_keyboard_grab_begin, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_keyboard_grab_end, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_touch_grab_begin, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_touch_grab_end, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)(wlr_seat_pointer_request_set_cursor_event * )>(_a, &qw_seat::notify_request_set_cursor, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)(wlr_seat_request_set_selection_event * )>(_a, &qw_seat::notify_request_set_selection, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_set_selection, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)(wlr_seat_request_set_primary_selection_event * )>(_a, &qw_seat::notify_request_set_primary_selection, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)()>(_a, &qw_seat::notify_set_primary_selection, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)(wlr_seat_request_start_drag_event * )>(_a, &qw_seat::notify_request_start_drag, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_seat::*)(wlr_drag * )>(_a, &qw_seat::notify_start_drag, 12))
            return;
    }
}

const QMetaObject *qw_seat::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_seat::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7qw_seatE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_seat,qw_seat>::qt_metacast(_clname);
}

int qw_seat::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_seat,qw_seat>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void qw_seat::notify_pointer_grab_begin()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qw_seat::notify_pointer_grab_end()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void qw_seat::notify_keyboard_grab_begin()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qw_seat::notify_keyboard_grab_end()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void qw_seat::notify_touch_grab_begin()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void qw_seat::notify_touch_grab_end()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void qw_seat::notify_request_set_cursor(wlr_seat_pointer_request_set_cursor_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void qw_seat::notify_request_set_selection(wlr_seat_request_set_selection_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void qw_seat::notify_set_selection()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void qw_seat::notify_request_set_primary_selection(wlr_seat_request_set_primary_selection_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void qw_seat::notify_set_primary_selection()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void qw_seat::notify_request_start_drag(wlr_seat_request_start_drag_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void qw_seat::notify_start_drag(wlr_drag * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN14qw_seat_clientE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_seat_client::qt_create_metaobjectdata<qt_meta_tag_ZN14qw_seat_clientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_seat_client"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_seat_client, qt_meta_tag_ZN14qw_seat_clientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_seat_client::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_seat_client,qw_seat_client>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_seat_clientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_seat_clientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14qw_seat_clientE_t>.metaTypes,
    nullptr
} };

void qw_seat_client::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_seat_client *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_seat_client::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_seat_client::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_seat_clientE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_seat_client,qw_seat_client>::qt_metacast(_clname);
}

int qw_seat_client::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_seat_client,qw_seat_client>::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
