/****************************************************************************
** Meta object code from reading C++ file 'qwbackend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/qwbackend.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwbackend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10qw_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_backend::qt_create_metaobjectdata<qt_meta_tag_ZN10qw_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_backend",
        "notify_new_output",
        "",
        "wlr_output*",
        "notify_new_input",
        "wlr_input_device*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_new_output'
        QtMocHelpers::SignalData<void(wlr_output *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_new_input'
        QtMocHelpers::SignalData<void(wlr_input_device *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_backend, qt_meta_tag_ZN10qw_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_backend,qw_backend>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10qw_backendE_t>.metaTypes,
    nullptr
} };

void qw_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_backend *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_new_output((*reinterpret_cast<std::add_pointer_t<wlr_output*>>(_a[1]))); break;
        case 1: _t->notify_new_input((*reinterpret_cast<std::add_pointer_t<wlr_input_device*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_backend::*)(wlr_output * )>(_a, &qw_backend::notify_new_output, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_backend::*)(wlr_input_device * )>(_a, &qw_backend::notify_new_input, 1))
            return;
    }
}

const QMetaObject *qw_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_backend,qw_backend>::qt_metacast(_clname);
}

int qw_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_backend,qw_backend>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void qw_backend::notify_new_output(wlr_output * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_backend::notify_new_input(wlr_input_device * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN16qw_multi_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_multi_backend::qt_create_metaobjectdata<qt_meta_tag_ZN16qw_multi_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_multi_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_multi_backend, qt_meta_tag_ZN16qw_multi_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_multi_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_multi_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_multi_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16qw_multi_backendE_t>.metaTypes,
    nullptr
} };

void qw_multi_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_multi_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_multi_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_multi_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_multi_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_multi_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14qw_drm_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_drm_backend::qt_create_metaobjectdata<qt_meta_tag_ZN14qw_drm_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_drm_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_drm_backend, qt_meta_tag_ZN14qw_drm_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_drm_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_drm_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_drm_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14qw_drm_backendE_t>.metaTypes,
    nullptr
} };

void qw_drm_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_drm_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_drm_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_drm_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_drm_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_drm_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18qw_wayland_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_wayland_backend::qt_create_metaobjectdata<qt_meta_tag_ZN18qw_wayland_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_wayland_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_wayland_backend, qt_meta_tag_ZN18qw_wayland_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_wayland_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qw_wayland_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qw_wayland_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qw_wayland_backendE_t>.metaTypes,
    nullptr
} };

void qw_wayland_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_wayland_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_wayland_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_wayland_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qw_wayland_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_wayland_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14qw_x11_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_x11_backend::qt_create_metaobjectdata<qt_meta_tag_ZN14qw_x11_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_x11_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_x11_backend, qt_meta_tag_ZN14qw_x11_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_x11_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_x11_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_x11_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14qw_x11_backendE_t>.metaTypes,
    nullptr
} };

void qw_x11_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_x11_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_x11_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_x11_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_x11_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_x11_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN19qw_libinput_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_libinput_backend::qt_create_metaobjectdata<qt_meta_tag_ZN19qw_libinput_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_libinput_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_libinput_backend, qt_meta_tag_ZN19qw_libinput_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_libinput_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_libinput_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_libinput_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19qw_libinput_backendE_t>.metaTypes,
    nullptr
} };

void qw_libinput_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_libinput_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_libinput_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_libinput_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_libinput_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_libinput_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN19qw_headless_backendE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_headless_backend::qt_create_metaobjectdata<qt_meta_tag_ZN19qw_headless_backendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_headless_backend"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_headless_backend, qt_meta_tag_ZN19qw_headless_backendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_headless_backend::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_backend::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_headless_backendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_headless_backendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19qw_headless_backendE_t>.metaTypes,
    nullptr
} };

void qw_headless_backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_headless_backend *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_headless_backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_headless_backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_headless_backendE_t>.strings))
        return static_cast<void*>(this);
    return qw_backend::qt_metacast(_clname);
}

int qw_headless_backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_backend::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
