/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x1,0xff,
  0x28,
  0xb5,0x2f,0xfd,0x60,0xaa,0x5,0xad,0xf,0x0,0x26,0xd7,0x43,0x1e,0x40,0x93,0x3a,
  0xbb,0xc8,0xca,0xc9,0xb1,0x65,0x73,0x3,0x81,0x30,0xb9,0xe3,0x7c,0xd6,0xbf,0x13,
  0xfc,0xa1,0xfa,0x24,0x97,0xb6,0x36,0x5c,0x1e,0xc7,0xe4,0x3b,0x0,0x3b,0x0,0x3c,
  0x0,0xf,0xd,0xe7,0xa5,0x1c,0x39,0x10,0x4,0xf5,0x5,0x1a,0x3c,0x7c,0xcb,0x19,
  0x3a,0xd1,0x5d,0x17,0xc0,0x5a,0x9e,0x1e,0x6a,0x3a,0xba,0xf2,0xc5,0x98,0xae,0x3d,
  0x2e,0xdf,0x6a,0xc3,0x75,0x9e,0x52,0x98,0x83,0x5d,0x51,0xf7,0xf4,0x29,0xce,0xb6,
  0x9e,0x3a,0x67,0x7a,0xe9,0x7e,0x8e,0x3c,0x77,0xd8,0xb4,0x57,0x51,0xd5,0x5a,0x33,
  0xe9,0x2d,0x61,0xb4,0x78,0xf9,0x13,0x6f,0x21,0xdb,0x29,0x2b,0x89,0x9a,0x28,0xfd,
  0xde,0x6f,0xd,0x6f,0x69,0x61,0x4c,0xb2,0x62,0x62,0xef,0x8d,0x6c,0xa7,0xa7,0x93,
  0xb7,0xda,0xc9,0xdb,0x7e,0xe0,0x2c,0xdc,0xb5,0x46,0xe0,0x1a,0x9c,0xf3,0xb6,0x27,
  0x0,0x10,0x3c,0x8d,0xc1,0xc6,0x2,0x3f,0x64,0x57,0x49,0x76,0xc5,0x9c,0xa3,0x63,
  0xd6,0x0,0xc5,0x6e,0xb7,0x58,0x10,0x8c,0x39,0xe7,0x5e,0x49,0x7c,0xcd,0x94,0xb0,
  0x6a,0xf,0x4b,0x6,0xdc,0xb6,0xe2,0xf,0xd9,0x56,0xdf,0xe8,0xa5,0x4f,0x1b,0xd9,
  0x1d,0x50,0xbc,0xf4,0x11,0x34,0xdc,0xbf,0xf6,0x8a,0x74,0x30,0xea,0x21,0xc2,0x84,
  0x26,0xb3,0x1c,0x8e,0x63,0x35,0xc5,0x20,0xd,0x79,0xe2,0xf4,0xe,0x3f,0x55,0x31,
  0x1d,0x94,0xdc,0x39,0xb2,0x5a,0x56,0x9c,0xcb,0xc4,0xfb,0xb9,0x49,0x56,0xdb,0x5b,
  0x4d,0x79,0xbb,0xdd,0x5d,0x3a,0x63,0xd,0x93,0x57,0x31,0x59,0xbd,0xf0,0x29,0xad,
  0xb6,0x5d,0x51,0xea,0x2a,0x4a,0x17,0xef,0xa4,0xee,0x2,0x6e,0xa8,0x81,0x8a,0xc,
  0x89,0x48,0xcd,0xa4,0xb0,0x30,0x1d,0x20,0x44,0xe8,0xb8,0xda,0x36,0x21,0xc,0x8a,
  0x22,0xc4,0x28,0x63,0x88,0x35,0x52,0x43,0x92,0xd4,0xa7,0x38,0x54,0x0,0x16,0x2f,
  0xa6,0xd0,0x37,0x36,0x1e,0x22,0xce,0xcc,0xde,0xa8,0x8,0xe5,0x4e,0x14,0xc9,0x3d,
  0x71,0x21,0x16,0x78,0x70,0x5b,0xee,0x9,0xf6,0xb6,0xcd,0xf5,0xe1,0x34,0x97,0xc2,
  0x64,0x76,0xb3,0x58,0x8c,0x10,0x1d,0xd2,0xba,0x37,0xc6,0xce,0xa0,0x22,0x71,0x2e,
  0x7a,0x9e,0x60,0x69,0x27,0x4e,0xdd,0x9e,0x2c,0x23,0x7e,0xe5,0x75,0x34,0x9,0x21,
  0x59,0xf,0xde,0x67,0xf0,0x27,0x17,0xfa,0xdc,0xc9,0x50,0x15,0x32,0xce,0x9a,0x8e,
  0x49,0x72,0x21,0x3b,0x33,0xb0,0xc9,0x95,0xf6,0x20,0xff,0x67,0x45,0x72,0x79,0xfc,
  0x20,0x2b,0x4d,0x43,0xbd,0x3d,0x52,0xa5,0xe6,0xcb,0x4c,0x2e,0x49,0xfc,0xc3,0xd6,
  0x81,0x9b,0x42,0x26,0xd2,0x46,0xd6,0x7e,0x3d,0x76,0xc7,0x20,0xd3,0x7b,0x62,0x5,
  0xfc,0xae,0x28,0x1b,0x25,0x68,0x6a,0x8b,0x32,0xfc,0xb1,0xab,0x93,0x4,0xb,0xc9,
  0x3c,0xa2,0xc7,0x83,0x9,0xaa,0x3e,0x3b,0x9d,0x5e,0xc6,0x40,0x12,0xa1,0xa4,0x72,
  0x2b,0x5b,0xad,0xf3,0x13,0xcb,0x4f,0x4,0x2d,0x50,0x1,0xe4,0x86,0x3,0xb4,0x6d,
  0xde,0xc1,0xf3,0xb4,0xab,0xbe,0xb4,0x81,0xe9,0xfd,0xd7,0xc3,0x55,0x1,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // Treeland
  0x0,0x8,
  0x8,0xbc,0x2f,0x24,
  0x0,0x54,
  0x0,0x72,0x0,0x65,0x0,0x65,0x0,0x6c,0x0,0x61,0x0,0x6e,0x0,0x64,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0xe1,0x28,0x8a,0x58,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0xe1,0x28,0x8a,0x58,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0x9e,0xe1,0x28,0x8a,0x58,
  // :/qt/qml/Treeland
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0x9e,0xe1,0x28,0x8a,0x58,
  // :/qt/qml/Treeland/qmldir
  0x0,0x0,0x0,0x2c,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0xe1,0x28,0x8a,0x58,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
