/****************************************************************************
** Meta object code from reading C++ file 'outputconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../outputconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'outputconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12OutputConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12OutputConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputConfig",
        "DConfigKeyList",
        "brightness;colorTemperature",
        "DConfigFileName",
        "org.deepin.dde.treeland.output",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "brightnessChanged",
        "colorTemperatureChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "brightnessIsDefaultValue",
        "colorTemperatureIsDefaultValue",
        "brightness",
        "colorTemperature"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'brightnessChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorTemperatureChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(18, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(19, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(20, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(21, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(22, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(23, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'brightnessIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(24, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'colorTemperatureIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(25, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'brightness'
        QtMocHelpers::PropertyData<double>(26, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'colorTemperature'
        QtMocHelpers::PropertyData<qlonglong>(27, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<OutputConfig, qt_meta_tag_ZN12OutputConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OutputConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12OutputConfigE_t>.metaTypes,
    nullptr
} };

void OutputConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->brightnessChanged(); break;
        case 4: _t->colorTemperatureChanged(); break;
        case 5: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->brightnessIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->colorTemperatureIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)(Dtk::Core::DConfig * )>(_a, &OutputConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)(const QString & , const QVariant & )>(_a, &OutputConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::brightnessChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::colorTemperatureChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<double*>(_v) = _t->brightness(); break;
        case 1: *reinterpret_cast<qlonglong*>(_v) = _t->colorTemperature(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBrightness(*reinterpret_cast<double*>(_v)); break;
        case 1: _t->setColorTemperature(*reinterpret_cast<qlonglong*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetBrightness(); break;
        case 1: _t->resetColorTemperature(); break;
        default: break;
        }
    }
}

const QMetaObject *OutputConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OutputConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void OutputConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OutputConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OutputConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OutputConfig::brightnessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OutputConfig::colorTemperatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
