/* Generated by wayland-scanner 1.25.0 */

/*
 * Copyright © 2012, 2013 Intel Corporation
 * Copyright © 2015, 2016 Jan Arne Petersen
 *
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface zwp_text_input_v2_interface;

static const struct wl_interface *text_input_unstable_v2_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	&wl_surface_interface,
	NULL,
	&wl_surface_interface,
	NULL,
	&wl_surface_interface,
	&zwp_text_input_v2_interface,
	&wl_seat_interface,
};

static const struct wl_message zwp_text_input_v2_requests[] = {
	{ "destroy", "", text_input_unstable_v2_types + 0 },
	{ "enable", "o", text_input_unstable_v2_types + 5 },
	{ "disable", "o", text_input_unstable_v2_types + 6 },
	{ "show_input_panel", "", text_input_unstable_v2_types + 0 },
	{ "hide_input_panel", "", text_input_unstable_v2_types + 0 },
	{ "set_surrounding_text", "sii", text_input_unstable_v2_types + 0 },
	{ "set_content_type", "uu", text_input_unstable_v2_types + 0 },
	{ "set_cursor_rectangle", "iiii", text_input_unstable_v2_types + 0 },
	{ "set_preferred_language", "s", text_input_unstable_v2_types + 0 },
	{ "update_state", "uu", text_input_unstable_v2_types + 0 },
};

static const struct wl_message zwp_text_input_v2_events[] = {
	{ "enter", "uo", text_input_unstable_v2_types + 7 },
	{ "leave", "uo", text_input_unstable_v2_types + 9 },
	{ "input_panel_state", "uiiii", text_input_unstable_v2_types + 0 },
	{ "preedit_string", "ss", text_input_unstable_v2_types + 0 },
	{ "preedit_styling", "uuu", text_input_unstable_v2_types + 0 },
	{ "preedit_cursor", "i", text_input_unstable_v2_types + 0 },
	{ "commit_string", "s", text_input_unstable_v2_types + 0 },
	{ "cursor_position", "ii", text_input_unstable_v2_types + 0 },
	{ "delete_surrounding_text", "uu", text_input_unstable_v2_types + 0 },
	{ "modifiers_map", "a", text_input_unstable_v2_types + 0 },
	{ "keysym", "uuuu", text_input_unstable_v2_types + 0 },
	{ "language", "s", text_input_unstable_v2_types + 0 },
	{ "text_direction", "u", text_input_unstable_v2_types + 0 },
	{ "configure_surrounding_text", "ii", text_input_unstable_v2_types + 0 },
	{ "input_method_changed", "uu", text_input_unstable_v2_types + 0 },
};

WL_EXPORT const struct wl_interface zwp_text_input_v2_interface = {
	"zwp_text_input_v2", 1,
	10, zwp_text_input_v2_requests,
	15, zwp_text_input_v2_events,
};

static const struct wl_message zwp_text_input_manager_v2_requests[] = {
	{ "destroy", "", text_input_unstable_v2_types + 0 },
	{ "get_text_input", "no", text_input_unstable_v2_types + 11 },
};

WL_EXPORT const struct wl_interface zwp_text_input_manager_v2_interface = {
	"zwp_text_input_manager_v2", 1,
	2, zwp_text_input_manager_v2_requests,
	0, NULL,
};

