/****************************************************************************
** Meta object code from reading C++ file 'qwinputdevice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwinputdevice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwinputdevice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15qw_input_deviceE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_input_device::qt_create_metaobjectdata<qt_meta_tag_ZN15qw_input_deviceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_input_device"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_input_device, qt_meta_tag_ZN15qw_input_deviceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_input_device::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_input_device,qw_input_device>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_input_deviceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_input_deviceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15qw_input_deviceE_t>.metaTypes,
    nullptr
} };

void qw_input_device::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_input_device *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_input_device::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_input_device::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_input_deviceE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_input_device,qw_input_device>::qt_metacast(_clname);
}

int qw_input_device::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_input_device,qw_input_device>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11qw_keyboardE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_keyboard::qt_create_metaobjectdata<qt_meta_tag_ZN11qw_keyboardE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_keyboard",
        "notify_key",
        "",
        "wlr_keyboard_key_event*",
        "notify_modifiers",
        "notify_keymap",
        "notify_repeat_info"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_key'
        QtMocHelpers::SignalData<void(wlr_keyboard_key_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_modifiers'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_keymap'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_repeat_info'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_keyboard, qt_meta_tag_ZN11qw_keyboardE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_keyboard::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11qw_keyboardE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11qw_keyboardE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11qw_keyboardE_t>.metaTypes,
    nullptr
} };

void qw_keyboard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_keyboard *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_key((*reinterpret_cast<std::add_pointer_t<wlr_keyboard_key_event*>>(_a[1]))); break;
        case 1: _t->notify_modifiers(); break;
        case 2: _t->notify_keymap(); break;
        case 3: _t->notify_repeat_info(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_keyboard::*)(wlr_keyboard_key_event * )>(_a, &qw_keyboard::notify_key, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_keyboard::*)()>(_a, &qw_keyboard::notify_modifiers, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_keyboard::*)()>(_a, &qw_keyboard::notify_keymap, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_keyboard::*)()>(_a, &qw_keyboard::notify_repeat_info, 3))
            return;
    }
}

const QMetaObject *qw_keyboard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_keyboard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11qw_keyboardE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_keyboard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void qw_keyboard::notify_key(wlr_keyboard_key_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_keyboard::notify_modifiers()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void qw_keyboard::notify_keymap()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qw_keyboard::notify_repeat_info()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN10qw_pointerE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_pointer::qt_create_metaobjectdata<qt_meta_tag_ZN10qw_pointerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_pointer",
        "notify_motion",
        "",
        "wlr_pointer_motion_event*",
        "notify_motion_absolute",
        "wlr_pointer_motion_absolute_event*",
        "notify_button",
        "wlr_pointer_button_event*",
        "notify_axis",
        "wlr_pointer_axis_event*",
        "notify_frame",
        "notify_swipe_begin",
        "wlr_pointer_swipe_begin_event*",
        "notify_swipe_update",
        "wlr_pointer_swipe_update_event*",
        "notify_swipe_end",
        "wlr_pointer_swipe_end_event*",
        "notify_pinch_begin",
        "wlr_pointer_pinch_begin_event*",
        "notify_pinch_update",
        "wlr_pointer_pinch_update_event*",
        "notify_pinch_end",
        "wlr_pointer_pinch_end_event*",
        "notify_hold_begin",
        "wlr_pointer_hold_begin_event*",
        "notify_hold_end",
        "wlr_pointer_hold_end_event*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_motion'
        QtMocHelpers::SignalData<void(wlr_pointer_motion_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_motion_absolute'
        QtMocHelpers::SignalData<void(wlr_pointer_motion_absolute_event *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_button'
        QtMocHelpers::SignalData<void(wlr_pointer_button_event *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_axis'
        QtMocHelpers::SignalData<void(wlr_pointer_axis_event *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'notify_frame'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_swipe_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_begin_event *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'notify_swipe_update'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_update_event *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Signal 'notify_swipe_end'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_end_event *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Signal 'notify_pinch_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_begin_event *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'notify_pinch_update'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_update_event *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Signal 'notify_pinch_end'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_end_event *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Signal 'notify_hold_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_hold_begin_event *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 2 },
        }}),
        // Signal 'notify_hold_end'
        QtMocHelpers::SignalData<void(wlr_pointer_hold_end_event *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_pointer, qt_meta_tag_ZN10qw_pointerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_pointer::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_pointerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_pointerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10qw_pointerE_t>.metaTypes,
    nullptr
} };

void qw_pointer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_pointer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_motion((*reinterpret_cast<std::add_pointer_t<wlr_pointer_motion_event*>>(_a[1]))); break;
        case 1: _t->notify_motion_absolute((*reinterpret_cast<std::add_pointer_t<wlr_pointer_motion_absolute_event*>>(_a[1]))); break;
        case 2: _t->notify_button((*reinterpret_cast<std::add_pointer_t<wlr_pointer_button_event*>>(_a[1]))); break;
        case 3: _t->notify_axis((*reinterpret_cast<std::add_pointer_t<wlr_pointer_axis_event*>>(_a[1]))); break;
        case 4: _t->notify_frame(); break;
        case 5: _t->notify_swipe_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_begin_event*>>(_a[1]))); break;
        case 6: _t->notify_swipe_update((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_update_event*>>(_a[1]))); break;
        case 7: _t->notify_swipe_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_end_event*>>(_a[1]))); break;
        case 8: _t->notify_pinch_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_begin_event*>>(_a[1]))); break;
        case 9: _t->notify_pinch_update((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_update_event*>>(_a[1]))); break;
        case 10: _t->notify_pinch_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_end_event*>>(_a[1]))); break;
        case 11: _t->notify_hold_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_hold_begin_event*>>(_a[1]))); break;
        case 12: _t->notify_hold_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_hold_end_event*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_motion_event * )>(_a, &qw_pointer::notify_motion, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_motion_absolute_event * )>(_a, &qw_pointer::notify_motion_absolute, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_button_event * )>(_a, &qw_pointer::notify_button, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_axis_event * )>(_a, &qw_pointer::notify_axis, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)()>(_a, &qw_pointer::notify_frame, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_swipe_begin_event * )>(_a, &qw_pointer::notify_swipe_begin, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_swipe_update_event * )>(_a, &qw_pointer::notify_swipe_update, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_swipe_end_event * )>(_a, &qw_pointer::notify_swipe_end, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_pinch_begin_event * )>(_a, &qw_pointer::notify_pinch_begin, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_pinch_update_event * )>(_a, &qw_pointer::notify_pinch_update, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_pinch_end_event * )>(_a, &qw_pointer::notify_pinch_end, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_hold_begin_event * )>(_a, &qw_pointer::notify_hold_begin, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_pointer::*)(wlr_pointer_hold_end_event * )>(_a, &qw_pointer::notify_hold_end, 12))
            return;
    }
}

const QMetaObject *qw_pointer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_pointer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10qw_pointerE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_pointer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void qw_pointer::notify_motion(wlr_pointer_motion_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_pointer::notify_motion_absolute(wlr_pointer_motion_absolute_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_pointer::notify_button(wlr_pointer_button_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_pointer::notify_axis(wlr_pointer_axis_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_pointer::notify_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void qw_pointer::notify_swipe_begin(wlr_pointer_swipe_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void qw_pointer::notify_swipe_update(wlr_pointer_swipe_update_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void qw_pointer::notify_swipe_end(wlr_pointer_swipe_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void qw_pointer::notify_pinch_begin(wlr_pointer_pinch_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void qw_pointer::notify_pinch_update(wlr_pointer_pinch_update_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void qw_pointer::notify_pinch_end(wlr_pointer_pinch_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void qw_pointer::notify_hold_begin(wlr_pointer_hold_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void qw_pointer::notify_hold_end(wlr_pointer_hold_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN9qw_tabletE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_tablet::qt_create_metaobjectdata<qt_meta_tag_ZN9qw_tabletE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_tablet",
        "notify_axis",
        "",
        "wlr_tablet_tool_axis_event*",
        "notify_proximity",
        "wlr_tablet_tool_proximity_event*",
        "notify_tip",
        "wlr_tablet_tool_tip_event*",
        "notify_button",
        "wlr_tablet_tool_button_event*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_axis'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_axis_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_proximity'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_proximity_event *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_tip'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_tip_event *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_button'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_button_event *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_tablet, qt_meta_tag_ZN9qw_tabletE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_tablet::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_tabletE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_tabletE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9qw_tabletE_t>.metaTypes,
    nullptr
} };

void qw_tablet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_tablet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_axis((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_axis_event*>>(_a[1]))); break;
        case 1: _t->notify_proximity((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_proximity_event*>>(_a[1]))); break;
        case 2: _t->notify_tip((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_tip_event*>>(_a[1]))); break;
        case 3: _t->notify_button((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_button_event*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_tablet::*)(wlr_tablet_tool_axis_event * )>(_a, &qw_tablet::notify_axis, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet::*)(wlr_tablet_tool_proximity_event * )>(_a, &qw_tablet::notify_proximity, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet::*)(wlr_tablet_tool_tip_event * )>(_a, &qw_tablet::notify_tip, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet::*)(wlr_tablet_tool_button_event * )>(_a, &qw_tablet::notify_button, 3))
            return;
    }
}

const QMetaObject *qw_tablet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_tablet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_tabletE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_tablet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void qw_tablet::notify_axis(wlr_tablet_tool_axis_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_tablet::notify_proximity(wlr_tablet_tool_proximity_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_tablet::notify_tip(wlr_tablet_tool_tip_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_tablet::notify_button(wlr_tablet_tool_button_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN13qw_tablet_padE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_tablet_pad::qt_create_metaobjectdata<qt_meta_tag_ZN13qw_tablet_padE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_tablet_pad",
        "notify_button",
        "",
        "wlr_tablet_pad_button_event*",
        "notify_ring",
        "wlr_tablet_pad_ring_event*",
        "notify_strip",
        "wlr_tablet_pad_strip_event*",
        "notify_attach_tablet",
        "wlr_tablet_tool*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_button'
        QtMocHelpers::SignalData<void(wlr_tablet_pad_button_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_ring'
        QtMocHelpers::SignalData<void(wlr_tablet_pad_ring_event *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_strip'
        QtMocHelpers::SignalData<void(wlr_tablet_pad_strip_event *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_attach_tablet'
        QtMocHelpers::SignalData<void(wlr_tablet_tool *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_tablet_pad, qt_meta_tag_ZN13qw_tablet_padE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_tablet_pad::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_tablet_padE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_tablet_padE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13qw_tablet_padE_t>.metaTypes,
    nullptr
} };

void qw_tablet_pad::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_tablet_pad *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_button((*reinterpret_cast<std::add_pointer_t<wlr_tablet_pad_button_event*>>(_a[1]))); break;
        case 1: _t->notify_ring((*reinterpret_cast<std::add_pointer_t<wlr_tablet_pad_ring_event*>>(_a[1]))); break;
        case 2: _t->notify_strip((*reinterpret_cast<std::add_pointer_t<wlr_tablet_pad_strip_event*>>(_a[1]))); break;
        case 3: _t->notify_attach_tablet((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_tablet_pad::*)(wlr_tablet_pad_button_event * )>(_a, &qw_tablet_pad::notify_button, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet_pad::*)(wlr_tablet_pad_ring_event * )>(_a, &qw_tablet_pad::notify_ring, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet_pad::*)(wlr_tablet_pad_strip_event * )>(_a, &qw_tablet_pad::notify_strip, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_tablet_pad::*)(wlr_tablet_tool * )>(_a, &qw_tablet_pad::notify_attach_tablet, 3))
            return;
    }
}

const QMetaObject *qw_tablet_pad::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_tablet_pad::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_tablet_padE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_tablet_pad::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void qw_tablet_pad::notify_button(wlr_tablet_pad_button_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_tablet_pad::notify_ring(wlr_tablet_pad_ring_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_tablet_pad::notify_strip(wlr_tablet_pad_strip_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_tablet_pad::notify_attach_tablet(wlr_tablet_tool * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN9qw_switchE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_switch::qt_create_metaobjectdata<qt_meta_tag_ZN9qw_switchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_switch",
        "notify_toggle",
        "",
        "wlr_switch_toggle_event*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_toggle'
        QtMocHelpers::SignalData<void(wlr_switch_toggle_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_switch, qt_meta_tag_ZN9qw_switchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_switch::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_switchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_switchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9qw_switchE_t>.metaTypes,
    nullptr
} };

void qw_switch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_switch *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_toggle((*reinterpret_cast<std::add_pointer_t<wlr_switch_toggle_event*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_switch::*)(wlr_switch_toggle_event * )>(_a, &qw_switch::notify_toggle, 0))
            return;
    }
}

const QMetaObject *qw_switch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_switch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_switchE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_switch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void qw_switch::notify_toggle(wlr_switch_toggle_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN8qw_touchE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_touch::qt_create_metaobjectdata<qt_meta_tag_ZN8qw_touchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_touch",
        "notify_down",
        "",
        "wlr_touch_down_event*",
        "notify_up",
        "wlr_touch_up_event*",
        "notify_motion",
        "wlr_touch_motion_event*",
        "notify_cancel",
        "wlr_touch_cancel_event*",
        "notify_frame"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_down'
        QtMocHelpers::SignalData<void(wlr_touch_down_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_up'
        QtMocHelpers::SignalData<void(wlr_touch_up_event *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_motion'
        QtMocHelpers::SignalData<void(wlr_touch_motion_event *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_cancel'
        QtMocHelpers::SignalData<void(wlr_touch_cancel_event *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'notify_frame'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_touch, qt_meta_tag_ZN8qw_touchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_touch::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_input_device::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_touchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_touchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8qw_touchE_t>.metaTypes,
    nullptr
} };

void qw_touch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_touch *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_down((*reinterpret_cast<std::add_pointer_t<wlr_touch_down_event*>>(_a[1]))); break;
        case 1: _t->notify_up((*reinterpret_cast<std::add_pointer_t<wlr_touch_up_event*>>(_a[1]))); break;
        case 2: _t->notify_motion((*reinterpret_cast<std::add_pointer_t<wlr_touch_motion_event*>>(_a[1]))); break;
        case 3: _t->notify_cancel((*reinterpret_cast<std::add_pointer_t<wlr_touch_cancel_event*>>(_a[1]))); break;
        case 4: _t->notify_frame(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_touch::*)(wlr_touch_down_event * )>(_a, &qw_touch::notify_down, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_touch::*)(wlr_touch_up_event * )>(_a, &qw_touch::notify_up, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_touch::*)(wlr_touch_motion_event * )>(_a, &qw_touch::notify_motion, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_touch::*)(wlr_touch_cancel_event * )>(_a, &qw_touch::notify_cancel, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_touch::*)()>(_a, &qw_touch::notify_frame, 4))
            return;
    }
}

const QMetaObject *qw_touch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_touch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_touchE_t>.strings))
        return static_cast<void*>(this);
    return qw_input_device::qt_metacast(_clname);
}

int qw_touch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_input_device::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void qw_touch::notify_down(wlr_touch_down_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_touch::notify_up(wlr_touch_up_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_touch::notify_motion(wlr_touch_motion_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_touch::notify_cancel(wlr_touch_cancel_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_touch::notify_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
