/****************************************************************************
** Meta object code from reading C++ file 'qwxwaylandsurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwxwaylandsurface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwxwaylandsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19qw_xwayland_surfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_xwayland_surface::qt_create_metaobjectdata<qt_meta_tag_ZN19qw_xwayland_surfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_xwayland_surface",
        "notify_request_configure",
        "",
        "wlr_xwayland_surface_configure_event*",
        "notify_request_move",
        "notify_request_resize",
        "wlr_xwayland_resize_event*",
        "notify_request_minimize",
        "wlr_xwayland_minimize_event*",
        "notify_request_maximize",
        "notify_request_fullscreen",
        "notify_request_activate",
        "notify_set_title",
        "notify_set_class",
        "notify_set_role",
        "notify_set_parent",
        "notify_set_startup_id",
        "notify_set_window_type",
        "notify_set_hints",
        "notify_set_decorations",
        "notify_set_override_redirect",
        "notify_set_geometry",
        "notify_ping_timeout",
        "notify_associate",
        "notify_dissociate"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_request_configure'
        QtMocHelpers::SignalData<void(wlr_xwayland_surface_configure_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_request_move'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_resize'
        QtMocHelpers::SignalData<void(wlr_xwayland_resize_event *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'notify_request_minimize'
        QtMocHelpers::SignalData<void(wlr_xwayland_minimize_event *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'notify_request_maximize'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_fullscreen'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_activate'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_title'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_class'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_role'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_parent'
        QtMocHelpers::SignalData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_startup_id'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_window_type'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_hints'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_decorations'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_override_redirect'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_set_geometry'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_ping_timeout'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_associate'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_dissociate'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_xwayland_surface, qt_meta_tag_ZN19qw_xwayland_surfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_xwayland_surface::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_xwayland_surface,qw_xwayland_surface>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_xwayland_surfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_xwayland_surfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19qw_xwayland_surfaceE_t>.metaTypes,
    nullptr
} };

void qw_xwayland_surface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_xwayland_surface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_request_configure((*reinterpret_cast<std::add_pointer_t<wlr_xwayland_surface_configure_event*>>(_a[1]))); break;
        case 1: _t->notify_request_move(); break;
        case 2: _t->notify_request_resize((*reinterpret_cast<std::add_pointer_t<wlr_xwayland_resize_event*>>(_a[1]))); break;
        case 3: _t->notify_request_minimize((*reinterpret_cast<std::add_pointer_t<wlr_xwayland_minimize_event*>>(_a[1]))); break;
        case 4: _t->notify_request_maximize(); break;
        case 5: _t->notify_request_fullscreen(); break;
        case 6: _t->notify_request_activate(); break;
        case 7: _t->notify_set_title(); break;
        case 8: _t->notify_set_class(); break;
        case 9: _t->notify_set_role(); break;
        case 10: _t->notify_set_parent(); break;
        case 11: _t->notify_set_startup_id(); break;
        case 12: _t->notify_set_window_type(); break;
        case 13: _t->notify_set_hints(); break;
        case 14: _t->notify_set_decorations(); break;
        case 15: _t->notify_set_override_redirect(); break;
        case 16: _t->notify_set_geometry(); break;
        case 17: _t->notify_ping_timeout(); break;
        case 18: _t->notify_associate(); break;
        case 19: _t->notify_dissociate(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)(wlr_xwayland_surface_configure_event * )>(_a, &qw_xwayland_surface::notify_request_configure, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_request_move, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)(wlr_xwayland_resize_event * )>(_a, &qw_xwayland_surface::notify_request_resize, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)(wlr_xwayland_minimize_event * )>(_a, &qw_xwayland_surface::notify_request_minimize, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_request_maximize, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_request_fullscreen, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_request_activate, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_title, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_class, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_role, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_parent, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_startup_id, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_window_type, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_hints, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_decorations, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_override_redirect, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_set_geometry, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_ping_timeout, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_associate, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xwayland_surface::*)()>(_a, &qw_xwayland_surface::notify_dissociate, 19))
            return;
    }
}

const QMetaObject *qw_xwayland_surface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_xwayland_surface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qw_xwayland_surfaceE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_xwayland_surface,qw_xwayland_surface>::qt_metacast(_clname);
}

int qw_xwayland_surface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_xwayland_surface,qw_xwayland_surface>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}

// SIGNAL 0
void qw_xwayland_surface::notify_request_configure(wlr_xwayland_surface_configure_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_xwayland_surface::notify_request_move()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void qw_xwayland_surface::notify_request_resize(wlr_xwayland_resize_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_xwayland_surface::notify_request_minimize(wlr_xwayland_minimize_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_xwayland_surface::notify_request_maximize()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void qw_xwayland_surface::notify_request_fullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void qw_xwayland_surface::notify_request_activate()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void qw_xwayland_surface::notify_set_title()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void qw_xwayland_surface::notify_set_class()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void qw_xwayland_surface::notify_set_role()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void qw_xwayland_surface::notify_set_parent()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void qw_xwayland_surface::notify_set_startup_id()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void qw_xwayland_surface::notify_set_window_type()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void qw_xwayland_surface::notify_set_hints()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void qw_xwayland_surface::notify_set_decorations()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void qw_xwayland_surface::notify_set_override_redirect()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void qw_xwayland_surface::notify_set_geometry()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void qw_xwayland_surface::notify_ping_timeout()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void qw_xwayland_surface::notify_associate()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void qw_xwayland_surface::notify_dissociate()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}
QT_WARNING_POP
