/****************************************************************************
** Meta object code from reading C++ file 'qwdatacontrolv1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwdatacontrolv1.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwdatacontrolv1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26qw_data_control_manager_v1E_t {};
} // unnamed namespace

template <> constexpr inline auto qw_data_control_manager_v1::qt_create_metaobjectdata<qt_meta_tag_ZN26qw_data_control_manager_v1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_data_control_manager_v1",
        "notify_new_device",
        "",
        "wlr_data_control_device_v1*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_new_device'
        QtMocHelpers::SignalData<void(wlr_data_control_device_v1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_data_control_manager_v1, qt_meta_tag_ZN26qw_data_control_manager_v1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_data_control_manager_v1::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_data_control_manager_v1,qw_data_control_manager_v1>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26qw_data_control_manager_v1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26qw_data_control_manager_v1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26qw_data_control_manager_v1E_t>.metaTypes,
    nullptr
} };

void qw_data_control_manager_v1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_data_control_manager_v1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_new_device((*reinterpret_cast<std::add_pointer_t<wlr_data_control_device_v1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_data_control_manager_v1::*)(wlr_data_control_device_v1 * )>(_a, &qw_data_control_manager_v1::notify_new_device, 0))
            return;
    }
}

const QMetaObject *qw_data_control_manager_v1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_data_control_manager_v1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26qw_data_control_manager_v1E_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_data_control_manager_v1,qw_data_control_manager_v1>::qt_metacast(_clname);
}

int qw_data_control_manager_v1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_data_control_manager_v1,qw_data_control_manager_v1>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void qw_data_control_manager_v1::notify_new_device(wlr_data_control_device_v1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
