/****************************************************************************
** Meta object code from reading C++ file 'activationmanagerinterfacev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/activation/activationmanagerinterfacev1.h"
#include "private/wserver_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'activationmanagerinterfacev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto ActivationManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ActivationManagerInterfaceV1",
        "activateRequested",
        "",
        "TokenDisposition",
        "disposition",
        "Waylib::Server::WSurface*",
        "surface",
        "Invalid",
        "Attention",
        "Active"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activateRequested'
        QtMocHelpers::SignalData<void(enum TokenDisposition, Waylib::Server::WSurface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'TokenDisposition'
        QtMocHelpers::EnumData<enum TokenDisposition>(3, 3, QMC::EnumIsScoped).add({
            {    7, TokenDisposition::Invalid },
            {    8, TokenDisposition::Attention },
            {    9, TokenDisposition::Active },
        }),
    };
    return QtMocHelpers::metaObjectData<ActivationManagerInterfaceV1, qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ActivationManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void ActivationManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActivationManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateRequested((*reinterpret_cast<std::add_pointer_t<enum TokenDisposition>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActivationManagerInterfaceV1::*)(TokenDisposition , Waylib::Server::WSurface * )>(_a, &ActivationManagerInterfaceV1::activateRequested, 0))
            return;
    }
}

const QMetaObject *ActivationManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ActivationManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28ActivationManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Waylib::Server::WServerInterface"))
        return static_cast< Waylib::Server::WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int ActivationManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void ActivationManagerInterfaceV1::activateRequested(TokenDisposition _t1, Waylib::Server::WSurface * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
