/****************************************************************************
** Meta object code from reading C++ file 'qwoutput.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwoutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwoutput.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9qw_outputE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_output::qt_create_metaobjectdata<qt_meta_tag_ZN9qw_outputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_output",
        "notify_frame",
        "",
        "notify_damage",
        "wlr_output_event_damage*",
        "notify_needs_frame",
        "notify_precommit",
        "wlr_output_event_precommit*",
        "notify_commit",
        "wlr_output_event_commit*",
        "notify_present",
        "wlr_output_event_present*",
        "notify_bind",
        "wlr_output_event_bind*",
        "notify_description",
        "notify_request_state",
        "wlr_output_event_request_state*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_frame'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_damage'
        QtMocHelpers::SignalData<void(wlr_output_event_damage *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'notify_needs_frame'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_precommit'
        QtMocHelpers::SignalData<void(wlr_output_event_precommit *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_commit'
        QtMocHelpers::SignalData<void(wlr_output_event_commit *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'notify_present'
        QtMocHelpers::SignalData<void(wlr_output_event_present *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 2 },
        }}),
        // Signal 'notify_bind'
        QtMocHelpers::SignalData<void(wlr_output_event_bind *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Signal 'notify_description'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_state'
        QtMocHelpers::SignalData<void(wlr_output_event_request_state *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_output, qt_meta_tag_ZN9qw_outputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_output::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_output,qw_output>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_outputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_outputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9qw_outputE_t>.metaTypes,
    nullptr
} };

void qw_output::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_output *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_frame(); break;
        case 1: _t->notify_damage((*reinterpret_cast<std::add_pointer_t<wlr_output_event_damage*>>(_a[1]))); break;
        case 2: _t->notify_needs_frame(); break;
        case 3: _t->notify_precommit((*reinterpret_cast<std::add_pointer_t<wlr_output_event_precommit*>>(_a[1]))); break;
        case 4: _t->notify_commit((*reinterpret_cast<std::add_pointer_t<wlr_output_event_commit*>>(_a[1]))); break;
        case 5: _t->notify_present((*reinterpret_cast<std::add_pointer_t<wlr_output_event_present*>>(_a[1]))); break;
        case 6: _t->notify_bind((*reinterpret_cast<std::add_pointer_t<wlr_output_event_bind*>>(_a[1]))); break;
        case 7: _t->notify_description(); break;
        case 8: _t->notify_request_state((*reinterpret_cast<std::add_pointer_t<wlr_output_event_request_state*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)()>(_a, &qw_output::notify_frame, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_damage * )>(_a, &qw_output::notify_damage, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)()>(_a, &qw_output::notify_needs_frame, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_precommit * )>(_a, &qw_output::notify_precommit, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_commit * )>(_a, &qw_output::notify_commit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_present * )>(_a, &qw_output::notify_present, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_bind * )>(_a, &qw_output::notify_bind, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)()>(_a, &qw_output::notify_description, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_output::*)(wlr_output_event_request_state * )>(_a, &qw_output::notify_request_state, 8))
            return;
    }
}

const QMetaObject *qw_output::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_output::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_outputE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_output,qw_output>::qt_metacast(_clname);
}

int qw_output::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_output,qw_output>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void qw_output::notify_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qw_output::notify_damage(wlr_output_event_damage * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_output::notify_needs_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qw_output::notify_precommit(wlr_output_event_precommit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_output::notify_commit(wlr_output_event_commit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void qw_output::notify_present(wlr_output_event_present * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void qw_output::notify_bind(wlr_output_event_bind * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void qw_output::notify_description()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void qw_output::notify_request_state(wlr_output_event_request_state * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
