/****************************************************************************
** Meta object code from reading C++ file 'qwscene.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwscene.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13qw_scene_nodeE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_node::qt_create_metaobjectdata<qt_meta_tag_ZN13qw_scene_nodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_node"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_node, qt_meta_tag_ZN13qw_scene_nodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_node::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_scene_node,qw_scene_node>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_nodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_nodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13qw_scene_nodeE_t>.metaTypes,
    nullptr
} };

void qw_scene_node::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_node *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene_node::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_node::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_nodeE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_scene_node,qw_scene_node>::qt_metacast(_clname);
}

int qw_scene_node::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_scene_node,qw_scene_node>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16qw_scene_surfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_surface::qt_create_metaobjectdata<qt_meta_tag_ZN16qw_scene_surfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_surface"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_surface, qt_meta_tag_ZN16qw_scene_surfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_surface::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_scene_surface,qw_scene_surface>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_scene_surfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_scene_surfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16qw_scene_surfaceE_t>.metaTypes,
    nullptr
} };

void qw_scene_surface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_surface *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene_surface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_surface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qw_scene_surfaceE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_scene_surface,qw_scene_surface>::qt_metacast(_clname);
}

int qw_scene_surface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_scene_surface,qw_scene_surface>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15qw_scene_outputE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_output::qt_create_metaobjectdata<qt_meta_tag_ZN15qw_scene_outputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_output"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_output, qt_meta_tag_ZN15qw_scene_outputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_output::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_scene_output,qw_scene_output>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_outputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_outputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15qw_scene_outputE_t>.metaTypes,
    nullptr
} };

void qw_scene_output::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_output *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene_output::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_output::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_outputE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_scene_output,qw_scene_output>::qt_metacast(_clname);
}

int qw_scene_output::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_scene_output,qw_scene_output>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13qw_scene_treeE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_tree::qt_create_metaobjectdata<qt_meta_tag_ZN13qw_scene_treeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_tree"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_tree, qt_meta_tag_ZN13qw_scene_treeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_tree::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_scene_node::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_treeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_treeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13qw_scene_treeE_t>.metaTypes,
    nullptr
} };

void qw_scene_tree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_tree *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene_tree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_tree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_treeE_t>.strings))
        return static_cast<void*>(this);
    return qw_scene_node::qt_metacast(_clname);
}

int qw_scene_tree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_scene_node::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN8qw_sceneE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene::qt_create_metaobjectdata<qt_meta_tag_ZN8qw_sceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene, qt_meta_tag_ZN8qw_sceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_scene_tree::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_sceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_sceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8qw_sceneE_t>.metaTypes,
    nullptr
} };

void qw_scene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8qw_sceneE_t>.strings))
        return static_cast<void*>(this);
    return qw_scene_tree::qt_metacast(_clname);
}

int qw_scene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_scene_tree::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13qw_scene_rectE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_rect::qt_create_metaobjectdata<qt_meta_tag_ZN13qw_scene_rectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_rect"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_rect, qt_meta_tag_ZN13qw_scene_rectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_rect::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_scene_node::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_rectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_rectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13qw_scene_rectE_t>.metaTypes,
    nullptr
} };

void qw_scene_rect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_rect *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *qw_scene_rect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_rect::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13qw_scene_rectE_t>.strings))
        return static_cast<void*>(this);
    return qw_scene_node::qt_metacast(_clname);
}

int qw_scene_rect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_scene_node::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15qw_scene_bufferE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_scene_buffer::qt_create_metaobjectdata<qt_meta_tag_ZN15qw_scene_bufferE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_scene_buffer",
        "notify_outputs_update",
        "",
        "wlr_scene_outputs_update_event*",
        "notify_output_enter",
        "wlr_scene_output*",
        "notify_output_leave",
        "notify_output_sample",
        "wlr_scene_output_sample_event*",
        "notify_frame_done",
        "timespec*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_outputs_update'
        QtMocHelpers::SignalData<void(wlr_scene_outputs_update_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_output_enter'
        QtMocHelpers::SignalData<void(wlr_scene_output *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_output_leave'
        QtMocHelpers::SignalData<void(wlr_scene_output *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_output_sample'
        QtMocHelpers::SignalData<void(wlr_scene_output_sample_event *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'notify_frame_done'
        QtMocHelpers::SignalData<void(timespec *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_scene_buffer, qt_meta_tag_ZN15qw_scene_bufferE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_scene_buffer::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_scene_node::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_bufferE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_bufferE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15qw_scene_bufferE_t>.metaTypes,
    nullptr
} };

void qw_scene_buffer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_scene_buffer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_outputs_update((*reinterpret_cast<std::add_pointer_t<wlr_scene_outputs_update_event*>>(_a[1]))); break;
        case 1: _t->notify_output_enter((*reinterpret_cast<std::add_pointer_t<wlr_scene_output*>>(_a[1]))); break;
        case 2: _t->notify_output_leave((*reinterpret_cast<std::add_pointer_t<wlr_scene_output*>>(_a[1]))); break;
        case 3: _t->notify_output_sample((*reinterpret_cast<std::add_pointer_t<wlr_scene_output_sample_event*>>(_a[1]))); break;
        case 4: _t->notify_frame_done((*reinterpret_cast<std::add_pointer_t<timespec*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_scene_buffer::*)(wlr_scene_outputs_update_event * )>(_a, &qw_scene_buffer::notify_outputs_update, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_scene_buffer::*)(wlr_scene_output * )>(_a, &qw_scene_buffer::notify_output_enter, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_scene_buffer::*)(wlr_scene_output * )>(_a, &qw_scene_buffer::notify_output_leave, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_scene_buffer::*)(wlr_scene_output_sample_event * )>(_a, &qw_scene_buffer::notify_output_sample, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_scene_buffer::*)(timespec * )>(_a, &qw_scene_buffer::notify_frame_done, 4))
            return;
    }
}

const QMetaObject *qw_scene_buffer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_scene_buffer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_scene_bufferE_t>.strings))
        return static_cast<void*>(this);
    return qw_scene_node::qt_metacast(_clname);
}

int qw_scene_buffer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_scene_node::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void qw_scene_buffer::notify_outputs_update(wlr_scene_outputs_update_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_scene_buffer::notify_output_enter(wlr_scene_output * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_scene_buffer::notify_output_leave(wlr_scene_output * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_scene_buffer::notify_output_sample(wlr_scene_output_sample_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_scene_buffer::notify_frame_done(timespec * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
