/****************************************************************************
** Meta object code from reading C++ file 'woutputlayout.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/woutputlayout.h"
#include "private/wserver_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputlayout.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputLayout::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputLayout",
        "outputAdded",
        "",
        "Waylib::Server::WOutput*",
        "output",
        "outputRemoved",
        "outputsChanged",
        "implicitWidthChanged",
        "implicitHeightChanged",
        "implicitWidth",
        "implicitHeight",
        "Layer",
        "Background",
        "Cursor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'outputAdded'
        QtMocHelpers::SignalData<void(Waylib::Server::WOutput *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'outputRemoved'
        QtMocHelpers::SignalData<void(Waylib::Server::WOutput *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'outputsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'implicitWidthChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'implicitHeightChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'implicitWidth'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags, 3),
        // property 'implicitHeight'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Layer'
        QtMocHelpers::EnumData<enum Layer>(11, 11, QMC::EnumIsScoped).add({
            {   12, Layer::Background },
            {   13, Layer::Cursor },
        }),
    };
    return QtMocHelpers::metaObjectData<WOutputLayout, qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutputLayout::staticMetaObject = { {
    QMetaObject::SuperData::link<WWrapObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputLayout::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputLayout *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->outputAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WOutput*>>(_a[1]))); break;
        case 1: _t->outputRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WOutput*>>(_a[1]))); break;
        case 2: _t->outputsChanged(); break;
        case 3: _t->implicitWidthChanged(); break;
        case 4: _t->implicitHeightChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputLayout::*)(Waylib::Server::WOutput * )>(_a, &WOutputLayout::outputAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayout::*)(Waylib::Server::WOutput * )>(_a, &WOutputLayout::outputRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayout::*)()>(_a, &WOutputLayout::outputsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayout::*)()>(_a, &WOutputLayout::implicitWidthChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayout::*)()>(_a, &WOutputLayout::implicitHeightChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->implicitWidth(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->implicitHeight(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputLayout::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputLayout::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WOutputLayoutE_t>.strings))
        return static_cast<void*>(this);
    return WWrapObject::qt_metacast(_clname);
}

int Waylib::Server::WOutputLayout::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WWrapObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputLayout::outputAdded(Waylib::Server::WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Waylib::Server::WOutputLayout::outputRemoved(Waylib::Server::WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WOutputLayout::outputsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WOutputLayout::implicitWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutputLayout::implicitHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
