/****************************************************************************
** Meta object code from reading C++ file 'qwxdgshell.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwxdgshell.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwxdgshell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12qw_xdg_shellE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_xdg_shell::qt_create_metaobjectdata<qt_meta_tag_ZN12qw_xdg_shellE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_xdg_shell",
        "notify_new_surface",
        "",
        "wlr_xdg_surface*",
        "notify_new_toplevel",
        "wlr_xdg_toplevel*",
        "notify_new_popup",
        "wlr_xdg_popup*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_new_surface'
        QtMocHelpers::SignalData<void(wlr_xdg_surface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_new_toplevel'
        QtMocHelpers::SignalData<void(wlr_xdg_toplevel *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_new_popup'
        QtMocHelpers::SignalData<void(wlr_xdg_popup *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_xdg_shell, qt_meta_tag_ZN12qw_xdg_shellE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_xdg_shell::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_xdg_shell,qw_xdg_shell>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_shellE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_shellE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12qw_xdg_shellE_t>.metaTypes,
    nullptr
} };

void qw_xdg_shell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_xdg_shell *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_new_surface((*reinterpret_cast<std::add_pointer_t<wlr_xdg_surface*>>(_a[1]))); break;
        case 1: _t->notify_new_toplevel((*reinterpret_cast<std::add_pointer_t<wlr_xdg_toplevel*>>(_a[1]))); break;
        case 2: _t->notify_new_popup((*reinterpret_cast<std::add_pointer_t<wlr_xdg_popup*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_shell::*)(wlr_xdg_surface * )>(_a, &qw_xdg_shell::notify_new_surface, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_shell::*)(wlr_xdg_toplevel * )>(_a, &qw_xdg_shell::notify_new_toplevel, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_shell::*)(wlr_xdg_popup * )>(_a, &qw_xdg_shell::notify_new_popup, 2))
            return;
    }
}

const QMetaObject *qw_xdg_shell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_xdg_shell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_shellE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_xdg_shell,qw_xdg_shell>::qt_metacast(_clname);
}

int qw_xdg_shell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_xdg_shell,qw_xdg_shell>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void qw_xdg_shell::notify_new_surface(wlr_xdg_surface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_xdg_shell::notify_new_toplevel(wlr_xdg_toplevel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_xdg_shell::notify_new_popup(wlr_xdg_popup * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN14qw_xdg_surfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_xdg_surface::qt_create_metaobjectdata<qt_meta_tag_ZN14qw_xdg_surfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_xdg_surface",
        "notify_ping_timeout",
        "",
        "notify_new_popup",
        "wlr_xdg_popup*",
        "notify_configure",
        "wlr_xdg_surface_configure*",
        "notify_ack_configure"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_ping_timeout'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_new_popup'
        QtMocHelpers::SignalData<void(wlr_xdg_popup *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'notify_configure'
        QtMocHelpers::SignalData<void(wlr_xdg_surface_configure *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'notify_ack_configure'
        QtMocHelpers::SignalData<void(wlr_xdg_surface_configure *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_xdg_surface, qt_meta_tag_ZN14qw_xdg_surfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_xdg_surface::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_xdg_surface,qw_xdg_surface>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_xdg_surfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_xdg_surfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14qw_xdg_surfaceE_t>.metaTypes,
    nullptr
} };

void qw_xdg_surface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_xdg_surface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_ping_timeout(); break;
        case 1: _t->notify_new_popup((*reinterpret_cast<std::add_pointer_t<wlr_xdg_popup*>>(_a[1]))); break;
        case 2: _t->notify_configure((*reinterpret_cast<std::add_pointer_t<wlr_xdg_surface_configure*>>(_a[1]))); break;
        case 3: _t->notify_ack_configure((*reinterpret_cast<std::add_pointer_t<wlr_xdg_surface_configure*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_surface::*)()>(_a, &qw_xdg_surface::notify_ping_timeout, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_surface::*)(wlr_xdg_popup * )>(_a, &qw_xdg_surface::notify_new_popup, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_surface::*)(wlr_xdg_surface_configure * )>(_a, &qw_xdg_surface::notify_configure, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_surface::*)(wlr_xdg_surface_configure * )>(_a, &qw_xdg_surface::notify_ack_configure, 3))
            return;
    }
}

const QMetaObject *qw_xdg_surface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_xdg_surface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14qw_xdg_surfaceE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_xdg_surface,qw_xdg_surface>::qt_metacast(_clname);
}

int qw_xdg_surface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_xdg_surface,qw_xdg_surface>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void qw_xdg_surface::notify_ping_timeout()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qw_xdg_surface::notify_new_popup(wlr_xdg_popup * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_xdg_surface::notify_configure(wlr_xdg_surface_configure * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_xdg_surface::notify_ack_configure(wlr_xdg_surface_configure * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN12qw_xdg_popupE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_xdg_popup::qt_create_metaobjectdata<qt_meta_tag_ZN12qw_xdg_popupE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_xdg_popup",
        "notify_reposition",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_reposition'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_xdg_popup, qt_meta_tag_ZN12qw_xdg_popupE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_xdg_popup::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_xdg_popup,qw_xdg_popup>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_popupE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_popupE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12qw_xdg_popupE_t>.metaTypes,
    nullptr
} };

void qw_xdg_popup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_xdg_popup *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_reposition(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_popup::*)()>(_a, &qw_xdg_popup::notify_reposition, 0))
            return;
    }
}

const QMetaObject *qw_xdg_popup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_xdg_popup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12qw_xdg_popupE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_xdg_popup,qw_xdg_popup>::qt_metacast(_clname);
}

int qw_xdg_popup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_xdg_popup,qw_xdg_popup>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void qw_xdg_popup::notify_reposition()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN15qw_xdg_toplevelE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_xdg_toplevel::qt_create_metaobjectdata<qt_meta_tag_ZN15qw_xdg_toplevelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_xdg_toplevel",
        "notify_request_maximize",
        "",
        "notify_request_fullscreen",
        "notify_request_minimize",
        "notify_request_move",
        "wlr_xdg_toplevel_move_event*",
        "notify_request_resize",
        "wlr_xdg_toplevel_resize_event*",
        "notify_request_show_window_menu",
        "wlr_xdg_toplevel_show_window_menu_event*",
        "notify_set_parent",
        "wlr_xdg_toplevel*",
        "notify_set_title",
        "char*",
        "notify_set_app_id"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_request_maximize'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_fullscreen'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_minimize'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_request_move'
        QtMocHelpers::SignalData<void(wlr_xdg_toplevel_move_event *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Signal 'notify_request_resize'
        QtMocHelpers::SignalData<void(wlr_xdg_toplevel_resize_event *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'notify_request_show_window_menu'
        QtMocHelpers::SignalData<void(wlr_xdg_toplevel_show_window_menu_event *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'notify_set_parent'
        QtMocHelpers::SignalData<void(wlr_xdg_toplevel *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'notify_set_title'
        QtMocHelpers::SignalData<void(char *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Signal 'notify_set_app_id'
        QtMocHelpers::SignalData<void(char *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_xdg_toplevel, qt_meta_tag_ZN15qw_xdg_toplevelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_xdg_toplevel::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_xdg_toplevel,qw_xdg_toplevel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_xdg_toplevelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_xdg_toplevelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15qw_xdg_toplevelE_t>.metaTypes,
    nullptr
} };

void qw_xdg_toplevel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_xdg_toplevel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_request_maximize(); break;
        case 1: _t->notify_request_fullscreen(); break;
        case 2: _t->notify_request_minimize(); break;
        case 3: _t->notify_request_move((*reinterpret_cast<std::add_pointer_t<wlr_xdg_toplevel_move_event*>>(_a[1]))); break;
        case 4: _t->notify_request_resize((*reinterpret_cast<std::add_pointer_t<wlr_xdg_toplevel_resize_event*>>(_a[1]))); break;
        case 5: _t->notify_request_show_window_menu((*reinterpret_cast<std::add_pointer_t<wlr_xdg_toplevel_show_window_menu_event*>>(_a[1]))); break;
        case 6: _t->notify_set_parent((*reinterpret_cast<std::add_pointer_t<wlr_xdg_toplevel*>>(_a[1]))); break;
        case 7: _t->notify_set_title((*reinterpret_cast<std::add_pointer_t<char*>>(_a[1]))); break;
        case 8: _t->notify_set_app_id((*reinterpret_cast<std::add_pointer_t<char*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)()>(_a, &qw_xdg_toplevel::notify_request_maximize, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)()>(_a, &qw_xdg_toplevel::notify_request_fullscreen, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)()>(_a, &qw_xdg_toplevel::notify_request_minimize, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(wlr_xdg_toplevel_move_event * )>(_a, &qw_xdg_toplevel::notify_request_move, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(wlr_xdg_toplevel_resize_event * )>(_a, &qw_xdg_toplevel::notify_request_resize, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(wlr_xdg_toplevel_show_window_menu_event * )>(_a, &qw_xdg_toplevel::notify_request_show_window_menu, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(wlr_xdg_toplevel * )>(_a, &qw_xdg_toplevel::notify_set_parent, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(char * )>(_a, &qw_xdg_toplevel::notify_set_title, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_xdg_toplevel::*)(char * )>(_a, &qw_xdg_toplevel::notify_set_app_id, 8))
            return;
    }
}

const QMetaObject *qw_xdg_toplevel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_xdg_toplevel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qw_xdg_toplevelE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_xdg_toplevel,qw_xdg_toplevel>::qt_metacast(_clname);
}

int qw_xdg_toplevel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_xdg_toplevel,qw_xdg_toplevel>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void qw_xdg_toplevel::notify_request_maximize()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qw_xdg_toplevel::notify_request_fullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void qw_xdg_toplevel::notify_request_minimize()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qw_xdg_toplevel::notify_request_move(wlr_xdg_toplevel_move_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_xdg_toplevel::notify_request_resize(wlr_xdg_toplevel_resize_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void qw_xdg_toplevel::notify_request_show_window_menu(wlr_xdg_toplevel_show_window_menu_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void qw_xdg_toplevel::notify_set_parent(wlr_xdg_toplevel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void qw_xdg_toplevel::notify_set_title(char * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void qw_xdg_toplevel::notify_set_app_id(char * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
