/****************************************************************************
** Meta object code from reading C++ file 'woutputlayoutitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/woutputlayoutitem.h"
#include "woutput.h"
#include <wquickoutputlayout.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputlayoutitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputLayoutItem::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputLayoutItem",
        "QML.Element",
        "OutputLayoutItem",
        "outputsChanged",
        "",
        "enterOutput",
        "WOutput*",
        "output",
        "leaveOutput",
        "layoutChanged",
        "updateOutputs",
        "layout",
        "WQuickOutputLayout*",
        "outputs",
        "QList<WOutput*>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'outputsChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enterOutput'
        QtMocHelpers::SignalData<void(WOutput *)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'leaveOutput'
        QtMocHelpers::SignalData<void(WOutput *)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateOutputs'
        QtMocHelpers::SlotData<void()>(10, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'layout'
        QtMocHelpers::PropertyData<WQuickOutputLayout*>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'outputs'
        QtMocHelpers::PropertyData<QList<WOutput*>>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Resettable | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WOutputLayoutItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutputLayoutItem::staticMetaObject = { {
    QMetaObject::SuperData::link<WQuickObserver::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputLayoutItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputLayoutItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->outputsChanged(); break;
        case 1: _t->enterOutput((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 2: _t->leaveOutput((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 3: _t->layoutChanged(); break;
        case 4: _t->d_func()->updateOutputs(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputLayoutItem::*)()>(_a, &WOutputLayoutItem::outputsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayoutItem::*)(WOutput * )>(_a, &WOutputLayoutItem::enterOutput, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayoutItem::*)(WOutput * )>(_a, &WOutputLayoutItem::leaveOutput, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayoutItem::*)()>(_a, &WOutputLayoutItem::layoutChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WQuickOutputLayout**>(_v) = _t->layout(); break;
        case 1: *reinterpret_cast<QList<WOutput*>*>(_v) = _t->outputs(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLayout(*reinterpret_cast<WQuickOutputLayout**>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 1: _t->resetOutput(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputLayoutItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputLayoutItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WOutputLayoutItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return WQuickObserver::qt_metacast(_clname);
}

int Waylib::Server::WOutputLayoutItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WQuickObserver::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputLayoutItem::outputsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutputLayoutItem::enterOutput(WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WOutputLayoutItem::leaveOutput(WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Waylib::Server::WOutputLayoutItem::layoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
