// This file is autogenerated. Changes will be overwritten.
#include "EWIEGA46WW/moc_qwbackend.cpp"
#include "EWIEGA46WW/moc_qwdisplay.cpp"
#include "EWIEGA46WW/moc_qwobject.cpp"
#include "EWIEGA46WW/moc_qwsession.cpp"
#include "RZRAGMB46M/moc_qwallocator.cpp"
#include "RZRAGMB46M/moc_qwrenderer.cpp"
#include "GZRP3O7STM/moc_qwbuffer.cpp"
#include "GZRP3O7STM/moc_qwcompositor.cpp"
#include "GZRP3O7STM/moc_qwcontenttypev1.cpp"
#include "GZRP3O7STM/moc_qwcursor.cpp"
#include "GZRP3O7STM/moc_qwcursorshapev1.cpp"
#include "GZRP3O7STM/moc_qwdatacontrolv1.cpp"
#include "GZRP3O7STM/moc_qwdatadevice.cpp"
#include "GZRP3O7STM/moc_qwdrm.cpp"
#include "GZRP3O7STM/moc_qwdrmleasev1.cpp"
#include "GZRP3O7STM/moc_qwexportdmabufv1.cpp"
#include "GZRP3O7STM/moc_qwextdatacontrolv1.cpp"
#include "GZRP3O7STM/moc_qwextforeigntoplevellistv1.cpp"
#include "GZRP3O7STM/moc_qwextimagecapturesourcev1.cpp"
#include "GZRP3O7STM/moc_qwextimagecopycapturev1.cpp"
#include "GZRP3O7STM/moc_qwforeigntoplevelhandlev1.cpp"
#include "GZRP3O7STM/moc_qwfractionalscalemanagerv1.cpp"
#include "GZRP3O7STM/moc_qwgammacontorlv1.cpp"
#include "GZRP3O7STM/moc_qwidleinhibitv1.cpp"
#include "GZRP3O7STM/moc_qwinputdevice.cpp"
#include "GZRP3O7STM/moc_qwinputmethodv2.cpp"
#include "GZRP3O7STM/moc_qwkeyboardgroup.cpp"
#include "GZRP3O7STM/moc_qwkeyboardshortcutsinhibitv1.cpp"
#include "GZRP3O7STM/moc_qwlayershellv1.cpp"
#include "GZRP3O7STM/moc_qwlinuxdmabufv1.cpp"
#include "GZRP3O7STM/moc_qwoutput.cpp"
#include "GZRP3O7STM/moc_qwoutputlayer.cpp"
#include "GZRP3O7STM/moc_qwoutputlayout.cpp"
#include "GZRP3O7STM/moc_qwoutputmanagementv1.cpp"
#include "GZRP3O7STM/moc_qwoutputpowermanagementv1.cpp"
#include "GZRP3O7STM/moc_qwpointerconstraintsv1.cpp"
#include "GZRP3O7STM/moc_qwpointergesturesv1.cpp"
#include "GZRP3O7STM/moc_qwpresentation.cpp"
#include "GZRP3O7STM/moc_qwprimaryselection.cpp"
#include "GZRP3O7STM/moc_qwprimaryselectionv1.cpp"
#include "GZRP3O7STM/moc_qwrelativepointerv1.cpp"
#include "GZRP3O7STM/moc_qwscene.cpp"
#include "GZRP3O7STM/moc_qwscreencopyv1.cpp"
#include "GZRP3O7STM/moc_qwseat.cpp"
#include "GZRP3O7STM/moc_qwsecuritycontextmanagerv1.cpp"
#include "GZRP3O7STM/moc_qwsessionlockv1.cpp"
#include "GZRP3O7STM/moc_qwsubcompositor.cpp"
#include "GZRP3O7STM/moc_qwtabletv2.cpp"
#include "GZRP3O7STM/moc_qwtearingcontrolv1.cpp"
#include "GZRP3O7STM/moc_qwtextinputv3.cpp"
#include "GZRP3O7STM/moc_qwtransientseatv1.cpp"
#include "GZRP3O7STM/moc_qwviewporter.cpp"
#include "GZRP3O7STM/moc_qwvirtualkeyboardv1.cpp"
#include "GZRP3O7STM/moc_qwvirtualpointerv1.cpp"
#include "GZRP3O7STM/moc_qwxdgactivationv1.cpp"
#include "GZRP3O7STM/moc_qwxdgdecorationmanagerv1.cpp"
#include "GZRP3O7STM/moc_qwxdgforeignregistry.cpp"
#include "GZRP3O7STM/moc_qwxdgforeignv1.cpp"
#include "GZRP3O7STM/moc_qwxdgforeignv2.cpp"
#include "GZRP3O7STM/moc_qwxdgoutputv1.cpp"
#include "GZRP3O7STM/moc_qwxdgshell.cpp"
#include "GZRP3O7STM/moc_qwxwayland.cpp"
#include "GZRP3O7STM/moc_qwxwaylandserver.cpp"
#include "GZRP3O7STM/moc_qwxwaylandshellv1.cpp"
#include "GZRP3O7STM/moc_qwxwaylandsurface.cpp"
