/****************************************************************************
** Meta object code from reading C++ file 'qwcursor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../qwlroots/src/types/qwcursor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwcursor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9qw_cursorE_t {};
} // unnamed namespace

template <> constexpr inline auto qw_cursor::qt_create_metaobjectdata<qt_meta_tag_ZN9qw_cursorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qw_cursor",
        "notify_motion",
        "",
        "wlr_pointer_motion_event*",
        "notify_motion_absolute",
        "wlr_pointer_motion_absolute_event*",
        "notify_button",
        "wlr_pointer_button_event*",
        "notify_axis",
        "wlr_pointer_axis_event*",
        "notify_frame",
        "notify_swipe_begin",
        "wlr_pointer_swipe_begin_event*",
        "notify_swipe_update",
        "wlr_pointer_swipe_update_event*",
        "notify_swipe_end",
        "wlr_pointer_swipe_end_event*",
        "notify_pinch_begin",
        "wlr_pointer_pinch_begin_event*",
        "notify_pinch_update",
        "wlr_pointer_pinch_update_event*",
        "notify_pinch_end",
        "wlr_pointer_pinch_end_event*",
        "notify_hold_begin",
        "wlr_pointer_hold_begin_event*",
        "notify_hold_end",
        "wlr_pointer_hold_end_event*",
        "notify_touch_up",
        "wlr_touch_up_event*",
        "notify_touch_down",
        "wlr_touch_down_event*",
        "notify_touch_motion",
        "wlr_touch_motion_event*",
        "notify_touch_cancel",
        "wlr_touch_cancel_event*",
        "notify_touch_frame",
        "notify_tablet_tool_axis",
        "wlr_tablet_tool_axis_event*",
        "notify_tablet_tool_proximity",
        "wlr_tablet_tool_proximity_event*",
        "notify_tablet_tool_tip",
        "wlr_tablet_tool_tip_event*",
        "notify_tablet_tool_button",
        "wlr_tablet_tool_button_event*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'notify_motion'
        QtMocHelpers::SignalData<void(wlr_pointer_motion_event *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'notify_motion_absolute'
        QtMocHelpers::SignalData<void(wlr_pointer_motion_absolute_event *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'notify_button'
        QtMocHelpers::SignalData<void(wlr_pointer_button_event *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'notify_axis'
        QtMocHelpers::SignalData<void(wlr_pointer_axis_event *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 2 },
        }}),
        // Signal 'notify_frame'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_swipe_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_begin_event *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Signal 'notify_swipe_update'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_update_event *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Signal 'notify_swipe_end'
        QtMocHelpers::SignalData<void(wlr_pointer_swipe_end_event *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Signal 'notify_pinch_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_begin_event *)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 2 },
        }}),
        // Signal 'notify_pinch_update'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_update_event *)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Signal 'notify_pinch_end'
        QtMocHelpers::SignalData<void(wlr_pointer_pinch_end_event *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 2 },
        }}),
        // Signal 'notify_hold_begin'
        QtMocHelpers::SignalData<void(wlr_pointer_hold_begin_event *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 2 },
        }}),
        // Signal 'notify_hold_end'
        QtMocHelpers::SignalData<void(wlr_pointer_hold_end_event *)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 2 },
        }}),
        // Signal 'notify_touch_up'
        QtMocHelpers::SignalData<void(wlr_touch_up_event *)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 2 },
        }}),
        // Signal 'notify_touch_down'
        QtMocHelpers::SignalData<void(wlr_touch_down_event *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 2 },
        }}),
        // Signal 'notify_touch_motion'
        QtMocHelpers::SignalData<void(wlr_touch_motion_event *)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 2 },
        }}),
        // Signal 'notify_touch_cancel'
        QtMocHelpers::SignalData<void(wlr_touch_cancel_event *)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 2 },
        }}),
        // Signal 'notify_touch_frame'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notify_tablet_tool_axis'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_axis_event *)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 2 },
        }}),
        // Signal 'notify_tablet_tool_proximity'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_proximity_event *)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 2 },
        }}),
        // Signal 'notify_tablet_tool_tip'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_tip_event *)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 41, 2 },
        }}),
        // Signal 'notify_tablet_tool_button'
        QtMocHelpers::SignalData<void(wlr_tablet_tool_button_event *)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qw_cursor, qt_meta_tag_ZN9qw_cursorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qw_cursor::staticMetaObject = { {
    QMetaObject::SuperData::link<qw_object<wlr_cursor,qw_cursor>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_cursorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_cursorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9qw_cursorE_t>.metaTypes,
    nullptr
} };

void qw_cursor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qw_cursor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->notify_motion((*reinterpret_cast<std::add_pointer_t<wlr_pointer_motion_event*>>(_a[1]))); break;
        case 1: _t->notify_motion_absolute((*reinterpret_cast<std::add_pointer_t<wlr_pointer_motion_absolute_event*>>(_a[1]))); break;
        case 2: _t->notify_button((*reinterpret_cast<std::add_pointer_t<wlr_pointer_button_event*>>(_a[1]))); break;
        case 3: _t->notify_axis((*reinterpret_cast<std::add_pointer_t<wlr_pointer_axis_event*>>(_a[1]))); break;
        case 4: _t->notify_frame(); break;
        case 5: _t->notify_swipe_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_begin_event*>>(_a[1]))); break;
        case 6: _t->notify_swipe_update((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_update_event*>>(_a[1]))); break;
        case 7: _t->notify_swipe_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_swipe_end_event*>>(_a[1]))); break;
        case 8: _t->notify_pinch_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_begin_event*>>(_a[1]))); break;
        case 9: _t->notify_pinch_update((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_update_event*>>(_a[1]))); break;
        case 10: _t->notify_pinch_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_pinch_end_event*>>(_a[1]))); break;
        case 11: _t->notify_hold_begin((*reinterpret_cast<std::add_pointer_t<wlr_pointer_hold_begin_event*>>(_a[1]))); break;
        case 12: _t->notify_hold_end((*reinterpret_cast<std::add_pointer_t<wlr_pointer_hold_end_event*>>(_a[1]))); break;
        case 13: _t->notify_touch_up((*reinterpret_cast<std::add_pointer_t<wlr_touch_up_event*>>(_a[1]))); break;
        case 14: _t->notify_touch_down((*reinterpret_cast<std::add_pointer_t<wlr_touch_down_event*>>(_a[1]))); break;
        case 15: _t->notify_touch_motion((*reinterpret_cast<std::add_pointer_t<wlr_touch_motion_event*>>(_a[1]))); break;
        case 16: _t->notify_touch_cancel((*reinterpret_cast<std::add_pointer_t<wlr_touch_cancel_event*>>(_a[1]))); break;
        case 17: _t->notify_touch_frame(); break;
        case 18: _t->notify_tablet_tool_axis((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_axis_event*>>(_a[1]))); break;
        case 19: _t->notify_tablet_tool_proximity((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_proximity_event*>>(_a[1]))); break;
        case 20: _t->notify_tablet_tool_tip((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_tip_event*>>(_a[1]))); break;
        case 21: _t->notify_tablet_tool_button((*reinterpret_cast<std::add_pointer_t<wlr_tablet_tool_button_event*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_motion_event * )>(_a, &qw_cursor::notify_motion, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_motion_absolute_event * )>(_a, &qw_cursor::notify_motion_absolute, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_button_event * )>(_a, &qw_cursor::notify_button, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_axis_event * )>(_a, &qw_cursor::notify_axis, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)()>(_a, &qw_cursor::notify_frame, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_swipe_begin_event * )>(_a, &qw_cursor::notify_swipe_begin, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_swipe_update_event * )>(_a, &qw_cursor::notify_swipe_update, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_swipe_end_event * )>(_a, &qw_cursor::notify_swipe_end, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_pinch_begin_event * )>(_a, &qw_cursor::notify_pinch_begin, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_pinch_update_event * )>(_a, &qw_cursor::notify_pinch_update, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_pinch_end_event * )>(_a, &qw_cursor::notify_pinch_end, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_hold_begin_event * )>(_a, &qw_cursor::notify_hold_begin, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_pointer_hold_end_event * )>(_a, &qw_cursor::notify_hold_end, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_touch_up_event * )>(_a, &qw_cursor::notify_touch_up, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_touch_down_event * )>(_a, &qw_cursor::notify_touch_down, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_touch_motion_event * )>(_a, &qw_cursor::notify_touch_motion, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_touch_cancel_event * )>(_a, &qw_cursor::notify_touch_cancel, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)()>(_a, &qw_cursor::notify_touch_frame, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_tablet_tool_axis_event * )>(_a, &qw_cursor::notify_tablet_tool_axis, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_tablet_tool_proximity_event * )>(_a, &qw_cursor::notify_tablet_tool_proximity, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_tablet_tool_tip_event * )>(_a, &qw_cursor::notify_tablet_tool_tip, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (qw_cursor::*)(wlr_tablet_tool_button_event * )>(_a, &qw_cursor::notify_tablet_tool_button, 21))
            return;
    }
}

const QMetaObject *qw_cursor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qw_cursor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9qw_cursorE_t>.strings))
        return static_cast<void*>(this);
    return qw_object<wlr_cursor,qw_cursor>::qt_metacast(_clname);
}

int qw_cursor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qw_object<wlr_cursor,qw_cursor>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void qw_cursor::notify_motion(wlr_pointer_motion_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qw_cursor::notify_motion_absolute(wlr_pointer_motion_absolute_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qw_cursor::notify_button(wlr_pointer_button_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void qw_cursor::notify_axis(wlr_pointer_axis_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qw_cursor::notify_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void qw_cursor::notify_swipe_begin(wlr_pointer_swipe_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void qw_cursor::notify_swipe_update(wlr_pointer_swipe_update_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void qw_cursor::notify_swipe_end(wlr_pointer_swipe_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void qw_cursor::notify_pinch_begin(wlr_pointer_pinch_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void qw_cursor::notify_pinch_update(wlr_pointer_pinch_update_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void qw_cursor::notify_pinch_end(wlr_pointer_pinch_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void qw_cursor::notify_hold_begin(wlr_pointer_hold_begin_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void qw_cursor::notify_hold_end(wlr_pointer_hold_end_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void qw_cursor::notify_touch_up(wlr_touch_up_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void qw_cursor::notify_touch_down(wlr_touch_down_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void qw_cursor::notify_touch_motion(wlr_touch_motion_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void qw_cursor::notify_touch_cancel(wlr_touch_cancel_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void qw_cursor::notify_touch_frame()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void qw_cursor::notify_tablet_tool_axis(wlr_tablet_tool_axis_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void qw_cursor::notify_tablet_tool_proximity(wlr_tablet_tool_proximity_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void qw_cursor::notify_tablet_tool_tip(wlr_tablet_tool_tip_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void qw_cursor::notify_tablet_tool_button(wlr_tablet_tool_button_event * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
